//
//  Poisson.swift
//  Classe - Aquarium
//
//  Created by camtasia01 on 2014-10-21.
//  Copyright (c) 2014 Production sur support. All rights reserved.
//
//
//  ============================================================================================
//  À l'usage exclusif des étudiants et étudiantes de
//  Techniques d'Intégration Multimédia
//  du cégep de Saint-Jérôme.
//  --------------------------------------------------------------------------------------------
//  Il est interdit de reproduire, en tout ou en partie, à des fins commerciales,
//  le code source, les scènes, les éléments graphiques, les classes et
//  tout autre contenu du présent projet sans l’autorisation écrite de l'auteur.
//
//  Pour obtenir l’autorisation de reproduire ou d’utiliser, en tout ou en partie,
//  le présent projet, veuillez communiquer avec:
//
//  Alain Boudreault, aboudrea@cstj.qc.ca
//
//  ============================================================================================

import Foundation
import UIKit

protocol PoissonDelegate{
    func poissonAFaim(sender:Poisson)
    func poissonEnDangerDeMort(sender:Poisson)
    func poissonEstMort(sender:Poisson)
}

/**
  Classe permettant de créer un gentil petit poisson rouge...
*/
class Poisson:UIImageView {
    // Propriétés
    /// Une chaine décrivant le poisson.
    var nomPoisson:String
    /// Le nom du fichier qui contient l'image.
    var nomImage:String
    var force:Int
    let seuilDangerDeMort = 10
    var delegate:PoissonDelegate?

    var unTimer = NSTimer()
    let intervalleTemps:NSTimeInterval = 2

    // Étape 7
    var timerDeplacement = NSTimer()
    let intervalleTempsDeplacement:NSTimeInterval = 1/15
    var _largeurEcran:CGFloat = 0
    var _hauteurEcran:CGFloat = 0
    
    // Etape 7
    func determinerDimensionsEcran(){
        // Déterminer la résolution de l'appareil
        let screenRect:CGRect = UIScreen.mainScreen().bounds
        _largeurEcran = screenRect.width
        _hauteurEcran = screenRect.height
        println(screenRect)
    } // determinerDimensionsEcran
    
    
    // Constructeur
    /**
    Construire un poisson à partir d'un nom et d'une force
    */

    init(nomPoisson:String, force:Int) {
        println("Je suis le constructeur de Poisson.")
        self.nomPoisson  = nomPoisson
        self.force       = force
        self.nomImage    = "n/a"
        
        // Étape 3.3 - Appel de super.init()
        // Note:  Il faut renseigner les propriétés locales
        // avant l'appel de super.init()
        // Il faut utiliser le constructeur de la super classe
        // Il n'est pas possible d'utiliser un init de convenance de la super classe (à valider sous 6.1).
        // Le contructeur de UIImageView est:  init(image: UIImage!)
        // Par contre, il ne permet pas de préciser une taille pour l'image.
        // Nous allons donc faire appel au contructeur de la super classe de UIImageView C-A-D, UIView(frame: CGRect)
        //
        
        // Créer une View à la position (20,150) de dimension (100, 50)
        super.init(frame: CGRectMake(20, 150, 100, 50))
        image = UIImage(named: "p13-frame-1.gif")

        // Créer l'animation à partir d'une suite d'images
        var images:[UIImage] = []
        // Note: Il y a 8 images pour l'animation du poisson P13
        for i in 1...8 {
            images.append(UIImage(named:"p13-frame-\(i).gif")!)
        }
        
        animationImages = images  // Un tableau d'images pour l'animation
        animationDuration = 1.5   // La durée de l'animation
        startAnimating()          // Démarrer l'animation

        
    } // init()

    
    /**
    Méthode de gestion PoissonDelegate
    */
    func gererLeProtocoleDelegate(timer: NSTimer){
        //println("testerDelegate de \(nom)")
        
        let impact = Int(arc4random_uniform(5))
        force-=impact
        delegate?.poissonAFaim(self)
        
        if force < seuilDangerDeMort {
            delegate?.poissonEnDangerDeMort(self)
            stopAnimating()

        } // force < 10
        
        if force <= 0 {
            unTimer.invalidate()
            delegate?.poissonEstMort(self)
        } // force <= 0
        
    } // testerDelegate
    
    // Constructeur
    /**
    Construire un poisson animé à x,y hauteur et largeur et délégué
    */
    init(nomPoisson:String, nomImage:String, nbImages:Int, posX:CGFloat, posY:CGFloat, largeur:CGFloat, hauteur:CGFloat, leDelegate:PoissonDelegate) {
        println("Je suis le constructeur de \(nomPoisson).")
        self.nomPoisson  = nomPoisson
        self.force       = 20
        self.nomImage    = nomImage
        
        delegate         = leDelegate
        
        // Note:  Il faut renseigner les propriétés locales
        // avant l'appel super.init()
        super.init(frame: CGRectMake(posX , posY, largeur, hauteur))
        
        // Image sans anime - poisson mort
        image = UIImage(named: "mort.png")
        
        // Créer l'animation à partir d'une suite d'images
        var images:[UIImage] = []
        
        for i in 1...nbImages {
            images.append(UIImage(named: "\(nomImage)-frame-\(i).gif")!)
        } // for nbImages
        
        animationImages = images
        animationDuration = 1.5
        startAnimating()
        
        
        unTimer = NSTimer.scheduledTimerWithTimeInterval(intervalleTemps, target: self, selector: Selector("gererLeProtocoleDelegate:"), userInfo: nil, repeats: true)
        
        userInteractionEnabled = true
        
        timerDeplacement = NSTimer.scheduledTimerWithTimeInterval(intervalleTempsDeplacement, target: self, selector: Selector("deplacerObjet:"), userInfo: nil, repeats: true)

        determinerDimensionsEcran()
        
    } // init()
    
    func deplacerObjet(timer: NSTimer){
        
        let vitesse:CGFloat = 3
        let largeurImage = self.frame.width / 2
        
        // Note: Les poissons vont de droite à gauche, sauf pour p12
        let _direction:CGFloat =  nomImage == "p12" ? -1 : 1
        var position = self.center
        position.x = self.center.x - (vitesse * _direction)
        
        // Ajuster la position si le poisson sort de l'écran
        if (_direction == 1 && (position.x + largeurImage) < 0){
            // À la fin, ajuster avec la largeur du view
            position.x = _largeurEcran + largeurImage
        }
        if (_direction == -1 && (position.x - largeurImage) > _largeurEcran){
            // Au debut, ajuster avec la largeur du view
            position.x = -largeurImage
        }
        
        // Renseigner la nouvelle position du poisson
        self.center = position;
        
    } // deplacerObjet
    // Constructeur de convenance
    /**
    Construire un poisson pas de nom et de force zéro!
    */
    convenience init(rien:Int = 0){
        self.init(nomPoisson:"Pas de nom", force:0)
    } // convenience init()

    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    /**
    Méthode pour épuiser artificiellement le poisson
    */
    func epuiserLePoisson(force:Int){
        
        self.force -= force
        
        if self.force < 10 {
            println("Le poisson \(nomPoisson) est très très faible")
            delegate?.poissonAFaim(self)
        }
    } // epuiserLePoisson
    
    /**
    Méthode pour nourrir le poisson
    */
    func nourrirLePoisson(nourriture:Int){
        force = nourriture
        if force > seuilDangerDeMort {
            startAnimating()
        }
    } // nourrirLePoisson
    
    // Méthodes
    
    /**
    Je retourne une jolie description du poisson
    */
    func quiSuisJe() ->String {
        return "\n------------------\nJe suis le poisson \(nomPoisson) et j'ai une force de \(force)\n"
    } // quiSuisJe()
    
} // class Poisson