//
//  ViewController.swift
//  Classe - Aquarium
//
//  Created by Alain on 2014-10-11.
//  Copyright (c) 2014 Production sur support. All rights reserved.
//
//  ============================================================================================
//  À l'usage exclusif des étudiants et étudiantes de
//  Techniques d'Intégration Multimédia
//  du cégep de Saint-Jérôme.
//  --------------------------------------------------------------------------------------------
//  Il est interdit de reproduire, en tout ou en partie, à des fins commerciales,
//  le code source, les scènes, les éléments graphiques, les classes et
//  tout autre contenu du présent projet sans l’autorisation écrite de l'auteur.
//
//  Pour obtenir l’autorisation de reproduire ou d’utiliser, en tout ou en partie,
//  le présent projet, veuillez communiquer avec:
//
//  Alain Boudreault, aboudrea@cstj.qc.ca
//
//  ============================================================================================


import UIKit
import AVFoundation

class ViewController: UIViewController, PoissonDelegate{
    
    
    // ***********************************************************************
    // Définition des IBOutlet
    // ***********************************************************************
    @IBOutlet weak var labelNBPoissons: UILabel!
    @IBOutlet weak var niveauNourriture: UIProgressView!
    @IBOutlet weak var finPartie: UIView!

    // ***********************************************************************
    // Définition des propriétés de la classe ViewController
    // ***********************************************************************
    let maximumNourritureDisponible = 100
    let unePortionDeNourriture      = 20
    var nourritureDisponible        = 0
    let nbPoissonsDepart            = 6
    var nbPoissons                  = 0
    var modelesPoisson              = [["nom":"p11", "nbImages":"8"],["nom":"p12", "nbImages":"5"], ["nom":"p13", "nbImages":"8"]]

    var playerTrameSonoreAquarium       = AVAudioPlayer()
    
    
    // ***********************************************************************
    // Définition des IBAction
    // ***********************************************************************
    @IBAction func recommencer(sender: AnyObject) {
        println("recommencer")
        commencerLaPartie()
    } // recommencer
   
    @IBAction func ajouterNourriture(sender: AnyObject) {
        println("ajouterNourriture")
        // Ajouter à la nourriture disponible un poucentage de la quant actuelle.
        nourritureDisponible += Int(arc4random_uniform(UInt32(nourritureDisponible)))
        
        // S'assurer que la quant de nourriture ne dépasse pas le maximum
        nourritureDisponible = nourritureDisponible >  maximumNourritureDisponible ? maximumNourritureDisponible : nourritureDisponible
        actualiserProgressViewQuantNourriture()
    }  // ajouterNourriture
    

    // ***********************************************************************
    // Méthodes de la classe ViewController
    // ***********************************************************************

    
    func ChargerTramesSonores(){
        // Permettre la lecture en arrière plan.
        // AVAudioSession.sharedInstance().setCategory(AVAudioSessionCategoryPlayback, error: nil)
        // AVAudioSession.sharedInstance().setActive(true, error: nil)
        
        let URLTrameSonoreAquarium = NSURL(fileURLWithPath: NSBundle.mainBundle().pathForResource("eau", ofType: "mp3")!)
        
        var error:NSError?
        
        playerTrameSonoreAquarium = AVAudioPlayer(contentsOfURL: URLTrameSonoreAquarium, error: &error)
        playerTrameSonoreAquarium.prepareToPlay()     // Charger le fichier son en mémoire
        playerTrameSonoreAquarium.numberOfLoops = -1  // Boucle à l'infini avec une valeur négative
        playerTrameSonoreAquarium.volume = 0.2
        
    }  // ChargerTramesSonores
    
    
    // Étape 5 - Ajouter la méthode suivante:
    // Méthode exécutée lorsque l'écran est touché.
    // touch.view pointe sur la View qui a reçu l'événement
    override func touchesBegan(touches: NSSet, withEvent event: UIEvent) {
        // println(touches.anyObject())
        
        super.touchesBegan(touches, withEvent: event)
        let touch : UITouch = touches.anyObject() as UITouch
        
        /*
        if touch.view.isKindOfClass(Poisson)
        {
        }
        */
        
        if let poissonCourant = touch.view as? Poisson {
            println("C'est un poisson!")
            if nourritureDisponible >= unePortionDeNourriture {
                poissonCourant.nourrirLePoisson(unePortionDeNourriture)
                nourritureDisponible -= unePortionDeNourriture
                actualiserProgressViewQuantNourriture()
            }
        } // if touch.view as? Poisson
        
    } // touchesBegan
    
    override func viewDidLoad() {
        println("viewDidLoad")
        super.viewDidLoad()
        // Ajouter des bulles
        // TODO

        
        ChargerTramesSonores()
        
        // Préparer l'aquarium
        commencerLaPartie()

        let aquarium = Aquarium(individus: [
            PoissonVolant(nomPoisson: "Exocoetidae", force: 99, hauteurDuVol: 66),
            Poisson(nomPoisson: "Bob", force: 99)
            ],
            dimension:100)
        
        println(aquarium.quiSuisJe())
        
        
    } // viewDidLoad


    /**
      Préparer la scène et placer les poissons.
    */
    func commencerLaPartie(){
        println("commencerLaPartie")
        // Masquer la View de fin de la partie
        finPartie.hidden = true
        nourritureDisponible = maximumNourritureDisponible
        actualiserProgressViewQuantNourriture()
        nbPoissons = nbPoissonsDepart
        actualiserLabelNBPoissons()
        // Ajouter des poissons
        ajouterLesPoissons(nbPoissons)
        playerTrameSonoreAquarium.play()

    } // commencerLaPartie
    
    
    /**
      Réactualiser l'indicateur du nombres de poissons.
    */
    func actualiserLabelNBPoissons(){
        println("actualiserLabelNBPoissons")
        labelNBPoissons.text = "\(nbPoissons)"
    } // actualiserLabelNBPoissons

    /**
        Réactualiser l'indicateur quantité de nourriture.
    */
    func actualiserProgressViewQuantNourriture(){
        println("actualiserProgressViewQuantNourriture")
        niveauNourriture.progress = Float(nourritureDisponible) / 100.0
    } // actualiserQuantNourriture

    /** 
    Créer des poissons et les ajouter à la scène principale.
    */
    func ajouterLesPoissons(quantite:Int){
        println("ajouterLesPoissons")
        // TODO
        // Poisson 1, pour tester le protocole
        let x = Poisson(nomPoisson: "Poisson 1", nomImage: "p11", nbImages: 8, posX: 50, posY: 180, largeur: 100, hauteur: 50, leDelegate: self)
        x.epuiserLePoisson(99)
        view.addSubview(x)
        
        // Ou bien directement, sans passer par une variable.
        view.addSubview(Poisson(nomPoisson: "Poisson 2", nomImage: "p11", nbImages: 8, posX: 50, posY: 120, largeur: 75 , hauteur: 37, leDelegate: self))
        view.addSubview(Poisson(nomPoisson: "Poisson 3", nomImage: "p11", nbImages: 8, posX: 130, posY: 120, largeur: 100 , hauteur: 50, leDelegate: self))
        
        view.addSubview(Poisson(nomPoisson: "Poisson 4", nomImage: modelesPoisson[2]["nom"]!, nbImages: (modelesPoisson[2]["nbImages"]!).toInt()!, posX: 75, posY:400, largeur: 100 , hauteur: 50, leDelegate: self))
        view.addSubview(Poisson(nomPoisson: "Poisson 5", nomImage: modelesPoisson[2]["nom"]!, nbImages: (modelesPoisson[2]["nbImages"]!).toInt()!, posX: 160, posY: 450, largeur: 100 , hauteur: 50, leDelegate: self))
        
        view.addSubview(Poisson(nomPoisson: "Poisson 6", nomImage: modelesPoisson[1]["nom"]!, nbImages: (modelesPoisson[1]["nbImages"]!).toInt()!, posX: 220, posY: 290, largeur: 100 , hauteur: 60, leDelegate: self))
        
        
    } // ajouterLesPoissons
    
    
    override func didReceiveMemoryWarning() {
        println("didReceiveMemoryWarning")
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    } // didReceiveMemoryWarning

    
    ///TODO: Méthodes de protocoles
    func poissonAFaim(sender:Poisson) {
        let nourriture = Int(arc4random_uniform(5) + 1)
        
        // sender.nourrirLePoisson(nourriture)
        println("Le poisson \(sender.nomPoisson) d'une force de \(sender.force) a mangé \(nourriture * 2) granule(s)")
    }

    
    func poissonEnDangerDeMort(sender: Poisson) {
        println("\(sender.nomPoisson) a une force de \(sender.force) et il va mourrir de faim!")
    }
    
    func poissonEstMort(sender: Poisson) {
        println("\(sender.nomPoisson) est mort")
        sender.removeFromSuperview()
        nbPoissons--
        actualiserLabelNBPoissons()
    } // poissonEstMort
}  // Fin de la classe ViewController

